#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <errno.h>
#include "funcs.h"
#include "config.h"

void free_cfg(struct CONFIG *cfg) {
	char **var1[] = {&cfg->root, &cfg->css, &cfg->logo, &cfg->desc, &cfg->title, &cfg->link, &cfg->xml_file, &cfg->language, &cfg->date_fmt};
	char var2[] = {cfg->root_alloc, cfg->css_alloc, cfg->logo_alloc, cfg->desc_alloc, cfg->title_alloc, cfg->link_alloc, cfg->xml_file_alloc, cfg->language_alloc, cfg->date_fmt_alloc};

	for (size_t i = 0; i < sizeof(var2); i++)
		if (var2[i] && *(var1[i]))
			free(*(var1[i]));
}

const char *skip_space(const char *str) {
	const char *ptr = str;
	while (isspace(*ptr))
		ptr++;

	return ptr;
}

int add_str(const char *prog_name, char **dst, char *flag, const char *src) {
	if (!(*src))
		return 1;

	*dst = strdup(skip_space(src));
	if (*dst == NULL) {
		fprintf(stderr, "%s: strdup: %s\n", prog_name, strerror(errno));
		return 1;
	}

	*flag = 1;
	return 0;
}

int add_int(const char *prog_name, size_t line, size_t *dst, const char *src) {
	char *ptr = "";
	*dst = strtoul(skip_space(src), &ptr, 10);

	if (*ptr) {
		fprintf(stderr, "%s: incorrect number at line %zu: %s\n", prog_name, line, ptr);
		return 1;
	}

	return 0;
}

int parse_config(struct CONFIG *cfg, const char *cfg_path) {
	FILE *fp = file_open(cfg->prog_name, cfg_path, "r");
	if (fp == NULL)
		return 1;

	char *str = NULL;
	ssize_t bytes = 0;
	size_t n = 0;

	size_t line = 0;
	int ret = 0;

	while ((bytes = getline(&str, &n, fp)) > 0 && ret == 0) {
		line++;
		if (str[0] == '\0' || str[0] == '\n' || str[0] == '#')
			continue;

		char *ptr = strrchr(str, '\n');
		if (ptr != NULL)
			*ptr = '\0';

		if ((ptr = strstr(str, "ROOT")))
			ret = add_str(cfg->prog_name, &cfg->root, &cfg->root_alloc, ptr + sizeof("ROOT"));

		else if ((ptr = strstr(str, "CSS")))
			ret = add_str(cfg->prog_name, &cfg->css, &cfg->css_alloc, ptr + sizeof("CSS"));

		else if ((ptr = strstr(str, "LOGO")))
			ret = add_str(cfg->prog_name, &cfg->logo, &cfg->logo_alloc, ptr + sizeof("LOGO"));

		else if ((ptr = strstr(str, "DESC")))
			ret = add_str(cfg->prog_name, &cfg->desc, &cfg->desc_alloc, ptr + sizeof("DESC"));

		else if ((ptr = strstr(str, "TITLE")))
			ret = add_str(cfg->prog_name, &cfg->title, &cfg->title_alloc, ptr + sizeof("TITLE"));

		else if ((ptr = strstr(str, "LINK")))
			ret = add_str(cfg->prog_name, &cfg->link, &cfg->link_alloc, ptr + sizeof("LINK"));

		else if ((ptr = strstr(str, "XML_FILE")))
			ret = add_str(cfg->prog_name, &cfg->xml_file, &cfg->xml_file_alloc, ptr + sizeof("XML_FILE"));

		else if ((ptr = strstr(str, "LANGUAGE")))
			ret = add_str(cfg->prog_name, &cfg->language, &cfg->language_alloc, ptr + sizeof("LANGUAGE"));

		else if ((ptr = strstr(str, "DATE_FORMAT")))
			ret = add_str(cfg->prog_name, &cfg->date_fmt, &cfg->date_fmt_alloc, ptr + sizeof("DATE_FORMAT"));

		else if ((ptr = strstr(str, "POSTS_AT_PAGE")))
			ret = add_int(cfg->prog_name, line, &cfg->posts_at_page, ptr + sizeof("POSTS_AT_PAGE"));

		else if ((ptr = strstr(str, "MAX_POSTS")))
			ret = add_int(cfg->prog_name, line, &cfg->max_posts, ptr + sizeof("MAX_POSTS"));

		else {
			fprintf(stderr, "%s: syntax error on %zu: %s\n", cfg->prog_name, line, str);
			ret = 1;
		}
	}

	fclose(fp);
	if (str)
		free(str);

	if (ret) {
		fprintf(stderr, "%s: parser error\n", cfg->prog_name);
		free_cfg(cfg);
	}

	return ret;
}
